@@ -10,7 +10,7 @@ Author: Ilan Schnell
 """
 from bitarray._bitarray import _bitarray, bitdiff, bits2bytes, _sysinfo
 
-__version__ = '1.1.0'
+__version__ = '1.2.0'
 
 
 class bitarray(_bitarray):
@@ -21,24 +21,20 @@ the optional initial object, and endianness.
 If no initial object is provided, an empty bitarray (length zero) is created.
 The initial object may be of the following types:
 
-int
-    Create a bitarray of given integer length.  The initial values are
-    arbitrary.  If you want all values to be set, use the .setall() method.
+`int`: Create a bitarray of given integer length.  The initial values are
+arbitrary.  If you want all values to be set, use the .setall() method.
 
-string
-    Create bitarray from a string of '0's and '1's.
+`str`: Create bitarray from a string of `0` and `1`.
 
-list, tuple, iterable
-    Create bitarray from a sequence, each element in the sequence is
-    converted to a bit using its truth value.
+`list`, `tuple`, `iterable`: Create bitarray from a sequence, each
+element in the sequence is converted to a bit using its truth value.
 
-bitarray
-    Create bitarray from another bitarray.  This is done by copying the
-    memory holding the bitarray data, and is hence very fast.
+`bitarray`: Create bitarray from another bitarray.  This is done by
+copying the memory holding the bitarray data, and is hence very fast.
 
-The optional keyword arguments 'endian' specifies the bit endianness of the
+The optional keyword arguments `endian` specifies the bit endianness of the
 created bitarray object.
-Allowed values are 'big' and 'little' (default is 'big').
+Allowed values are the strings `big` and `little` (default is `big`).
 
 Note that setting the bit endianness only has an effect when accessing the
 machine representation of the bitarray, i.e. when using the methods: tofile,
@@ -48,7 +44,7 @@ fromfile, tobytes, frombytes."""
         """fromstring(str)
 
 Append from a string, interpreting the string as machine values.
-Deprecated since version 0.4.0, use ``frombytes()`` instead."""
+Deprecated since version 0.4.0, use `.frombytes()` instead."""
         return self.frombytes(string.encode())
 
     def tostring(self):
@@ -57,18 +53,9 @@ Deprecated since version 0.4.0, use ``frombytes()`` instead."""
 Return the string representing (machine values) of the bitarray.
 When the length of the bitarray is not a multiple of 8, the few remaining
 bits (1..7) are set to 0.
-Deprecated since version 0.4.0, use ``tobytes()`` instead."""
+Deprecated since version 0.4.0, use `.tobytes()` instead."""
         return self.tobytes().decode()
 
-    def __int__(self):
-        raise TypeError("int() cannot take bitarray as argument")
-
-    def __long__(self):
-        raise TypeError("long() cannot take bitarray as argument")
-
-    def __float__(self):
-        raise TypeError("float() cannot take bitarray as argument")
-
 
 class frozenbitarray(_bitarray):
     """frozenbitarray(initial=0, /, endian='big')
