@@ -44,13 +44,11 @@ typedef long long int idx_t;
    machine.  */
 typedef struct {
     PyObject_VAR_HEAD
-#ifdef WITH_BUFFER
-    int ob_exports;             /* how many buffer exports */
-#endif
     char *ob_item;
     Py_ssize_t allocated;       /* how many bytes allocated */
     idx_t nbits;                /* length of bitarray, i.e. elements */
     int endian;                 /* bit endianness of bitarray */
+    int ob_exports;             /* how many buffer exports */
     PyObject *weakreflist;      /* list of weak references */
 } bitarrayobject;
 
@@ -58,7 +56,7 @@ static PyTypeObject Bitarraytype;
 
 #define bitarray_Check(obj)  PyObject_TypeCheck(obj, &Bitarraytype)
 
-#define BITS(bytes)  (((idx_t) 8) * ((idx_t) (bytes)))
+#define BITS(bytes)  ((idx_t) (bytes) << 3)
 
 /* number of bytes necessary to store given bits */
 #define BYTES(bits)  (((bits) == 0) ? 0 : (((bits) - 1) / 8 + 1))
@@ -67,14 +65,22 @@ static PyTypeObject Bitarraytype;
 
 /* ------------ low level access to bits in bitarrayobject ------------- */
 
+#ifndef NDEBUG
+static int GETBIT(bitarrayobject *self, idx_t i) {
+    assert(0 <= i && i < self->nbits);
+    return ((self)->ob_item[(i) / 8] & BITMASK((self)->endian, i) ? 1 : 0);
+}
+#else
 #define GETBIT(self, i)  \
     ((self)->ob_item[(i) / 8] & BITMASK((self)->endian, i) ? 1 : 0)
+#endif
 
 static void
 setbit(bitarrayobject *self, idx_t i, int bit)
 {
     char *cp, mask;
 
+    assert(0 <= i && i < BITS(Py_SIZE(self)));
     mask = BITMASK(self->endian, i);
     cp = self->ob_item + i / 8;
     if (bit)
@@ -214,7 +220,7 @@ copy_n(bitarrayobject *self, idx_t a,
     if (self->endian == other->endian && a % 8 == 0 && b % 8 == 0 && n >= 8)
     {
         const Py_ssize_t bytes = (Py_ssize_t) n / 8;
-        const idx_t bits = bytes * 8;
+        const idx_t bits = BITS(bytes);
 
         assert(bits <= n && n < bits + 8);
         if (a <= b)
@@ -229,8 +235,9 @@ copy_n(bitarrayobject *self, idx_t a,
         return;
     }
 
-    /* The different type of looping is only relevant copying self to self,
-       i.e. when copying a piece of an bitarrayobject onto itself. */
+    /* The two different types of looping are only relevant when copying
+       self to self, i.e. when copying a piece of an bitarrayobject onto
+       itself. */
     if (a <= b) {
         for (i = 0; i < n; i++)             /* loop forward (delete) */
             setbit(self, i + a, GETBIT(other, i + b));
@@ -274,10 +281,10 @@ insert_n(bitarrayobject *self, idx_t start, idx_t n)
 static int
 setunused(bitarrayobject *self)
 {
-    idx_t i, n;
+    const idx_t n = BITS(Py_SIZE(self));
+    idx_t i;
     int res = 0;
 
-    n = BITS(Py_SIZE(self));
     for (i = self->nbits; i < n; i++) {
         setbit(self, i, 0);
         res++;
@@ -313,7 +320,7 @@ enum op_type {
     OP_xor,
 };
 
-/* perform bitwise operation */
+/* perform bitwise in-place operation */
 static int
 bitwise(bitarrayobject *self, PyObject *arg, enum op_type oper)
 {
@@ -367,11 +374,11 @@ setrange(bitarrayobject *self, idx_t start, idx_t stop, int val)
     if (stop >= start + 8) {
         const Py_ssize_t byte_start = BYTES(start);
         const Py_ssize_t byte_stop = (Py_ssize_t) stop / 8;
-        for (i = start; i < byte_start * 8; i++)
+        for (i = start; i < BITS(byte_start); i++)
             setbit(self, i, val);
         memset(self->ob_item + byte_start, val ? 0xff : 0x00,
                byte_stop - byte_start);
-        for (i = byte_stop * 8; i < stop; i++)
+        for (i = BITS(byte_stop); i < stop; i++)
             setbit(self, i, val);
     }
     else {
@@ -455,24 +462,21 @@ count(bitarrayobject *self, int vi, idx_t start, idx_t stop)
 
     if (stop >= start + 8) {
         const Py_ssize_t byte_start = BYTES(start);
-        const Py_ssize_t byte_stop = (Py_ssize_t) stop / 8;
+        const Py_ssize_t byte_stop = (Py_ssize_t) (stop / 8);
         Py_ssize_t j;
 
-        for (i = start; i < byte_start * 8; i++)
-            if (GETBIT(self, i))
-                res++;
+        for (i = start; i < BITS(byte_start); i++)
+            res += GETBIT(self, i);
         for (j = byte_start; j < byte_stop; j++) {
             c = self->ob_item[j];
             res += bitcount_lookup[c];
         }
-        for (i = byte_stop * 8; i < stop; i++)
-            if (GETBIT(self, i))
-                res++;
+        for (i = BITS(byte_stop); i < stop; i++)
+            res += GETBIT(self, i);
     }
     else {
         for (i = start; i < stop; i++)
-            if (GETBIT(self, i))
-                res++;
+            res += GETBIT(self, i);
     }
     return vi ? res : stop - start - res;
 }
@@ -939,8 +943,8 @@ PyDoc_STRVAR(length_doc,
 "length() -> int\n\
 \n\
 Return the length, i.e. number of bits stored in the bitarray.\n\
-This method is preferred over __len__ (used when typing ``len(a)``),\n\
-since __len__ will fail for a bitarray object with 2^31 or more elements\n\
+This method is preferred over `__len__` (used when typing `len(a)`),\n\
+since `__len__` will fail for a bitarray object with 2^31 or more elements\n\
 on a 32bit machine, whereas this method will return the correct value,\n\
 on 32bit and 64bit machines.");
 
@@ -1025,8 +1029,8 @@ bitarray_index(bitarrayobject *self, PyObject *args)
 PyDoc_STRVAR(index_doc,
 "index(value, start=0, stop=<end of array>, /) -> int\n\
 \n\
-Return index of the first occurrence of bool(value) in the bitarray.\n\
-Raises ValueError if the value is not present.");
+Return index of the first occurrence of `bool(value)` in the bitarray.\n\
+Raises `ValueError` if the value is not present.");
 
 
 static PyObject *
@@ -1169,7 +1173,7 @@ bitarray_endian(bitarrayobject *self)
 PyDoc_STRVAR(endian_doc,
 "endian() -> str\n\
 \n\
-Return the bit endianness as a string (either 'little' or 'big').");
+Return the bit endianness as a string (either `little` or `big`).");
 
 
 static PyObject *
@@ -1184,7 +1188,7 @@ bitarray_append(bitarrayobject *self, PyObject *v)
 PyDoc_STRVAR(append_doc,
 "append(item, /)\n\
 \n\
-Append the value bool(item) to the end of the bitarray.");
+Append the value `bool(item)` to the end of the bitarray.");
 
 
 static PyObject *
@@ -1356,7 +1360,7 @@ bitarray_setall(bitarrayobject *self, PyObject *v)
 PyDoc_STRVAR(setall_doc,
 "setall(value, /)\n\
 \n\
-Set all bits in the bitarray to bool(value).");
+Set all bits in the bitarray to `bool(value)`.");
 
 
 static PyObject *
@@ -1641,7 +1645,7 @@ bitarray_frombytes(bitarrayobject *self, PyObject *bytes)
     idx_t t, p;
 
     if (!PyBytes_Check(bytes)) {
-        PyErr_SetString(PyExc_TypeError, "byte string expected");
+        PyErr_SetString(PyExc_TypeError, "bytes expected");
         return NULL;
     }
 
@@ -1733,7 +1737,7 @@ static PyObject *
 bitarray_pack(bitarrayobject *self, PyObject *bytes)
 {
     if (!PyBytes_Check(bytes)) {
-        PyErr_SetString(PyExc_TypeError, "byte string expected");
+        PyErr_SetString(PyExc_TypeError, "bytes expected");
         return NULL;
     }
     if (extend_bytes(self, bytes, BYTES_RAW) < 0)
@@ -1746,7 +1750,7 @@ PyDoc_STRVAR(pack_doc,
 "pack(bytes, /)\n\
 \n\
 Extend the bitarray from bytes, where each byte corresponds to a single\n\
-bit.  The byte b'\\x00' maps to bit 0 and all other characters map to\n\
+bit.  The byte `b'\\x00'` maps to bit 0 and all other characters map to\n\
 bit 1.\n\
 This method, as well as the unpack method, are meant for efficient\n\
 transfer of data between bitarray objects to other python objects\n\
@@ -1802,9 +1806,9 @@ bitarray_insert(bitarrayobject *self, PyObject *args)
 }
 
 PyDoc_STRVAR(insert_doc,
-"insert(i, item, /)\n\
+"insert(index, value, /)\n\
 \n\
-Insert bool(item) into the bitarray before position i.");
+Insert `bool(value)` into the bitarray before index.");
 
 
 static PyObject *
@@ -1838,7 +1842,7 @@ PyDoc_STRVAR(pop_doc,
 "pop(index=-1, /) -> item\n\
 \n\
 Return the i-th (default last) element and delete it from the bitarray.\n\
-Raises IndexError if bitarray is empty or index is out of range.");
+Raises `IndexError` if bitarray is empty or index is out of range.");
 
 
 static PyObject *
@@ -1862,10 +1866,10 @@ bitarray_remove(bitarrayobject *self, PyObject *v)
 }
 
 PyDoc_STRVAR(remove_doc,
-"remove(item, /)\n\
+"remove(value, /)\n\
 \n\
-Remove the first occurrence of bool(item) in the bitarray.\n\
-Raises ValueError if item is not present.");
+Remove the first occurrence of `bool(value)` in the bitarray.\n\
+Raises `ValueError` if item is not present.");
 
 
 /* --------- special methods ----------- */
@@ -3006,7 +3010,7 @@ static PyTypeObject BitarrayIter_Type = {
 /********************* Bitarray Buffer Interface ************************/
 #ifdef WITH_BUFFER
 
-#if PY_MAJOR_VERSION == 2
+#if PY_MAJOR_VERSION == 2       /* old buffer protocol */
 static Py_ssize_t
 bitarray_buffer_getreadbuf(bitarrayobject *self,
                            Py_ssize_t index, const void **ptr)
@@ -3091,15 +3095,6 @@ static PyBufferProcs bitarray_as_buffer = {
 
 #endif  /* WITH_BUFFER */
 
-#if PY_MAJOR_VERSION == 2
-static long
-bitarray_hash(bitarrayobject *self)
-{
-    PyErr_SetString(PyExc_TypeError, "unhashable type");
-    return -1;
-}
-#endif
-
 /************************** Bitarray Type *******************************/
 
 static PyTypeObject Bitarraytype = {
@@ -3122,11 +3117,7 @@ static PyTypeObject Bitarraytype = {
     0,                                        /* tp_as_number*/
     0,                                        /* tp_as_sequence */
     0,                                        /* tp_as_mapping */
-#ifdef IS_PY3K
-    0,                                        /* tp_hash */
-#else
-    (hashfunc) bitarray_hash,                 /* tp_hash */
-#endif
+    PyObject_HashNotImplemented,              /* tp_hash */
     0,                                        /* tp_call */
     0,                                        /* tp_str */
     PyObject_GenericGetAttr,                  /* tp_getattro */
@@ -3202,7 +3193,8 @@ PyDoc_STRVAR(bitdiff_doc,
 \n\
 Return the difference between two bitarrays a and b.\n\
 This is function does the same as (a ^ b).count(), but is more memory\n\
-efficient, as no intermediate bitarray object gets created.");
+efficient, as no intermediate bitarray object gets created.\n\
+Deprecated since version 1.2.0, use `bitarray.util.count_xor()` instead.");
 
 
 static PyObject *
@@ -3249,7 +3241,12 @@ tuple(sizeof(void *),\n\
       sizeof(idx_t),\n\
       PY_SSIZE_T_MAX)");
 
-
+/*
+   In retrospect, I wish I had never added any modules functions here.
+   These, and possibly many others should be part of a separate utility
+   module.  Anyway, at this point (2019) it is too late to remove them,
+   so I will just leave them here, but not any new ones.
+*/
 static PyMethodDef module_functions[] = {
     {"bitdiff",    (PyCFunction) bitdiff,    METH_VARARGS, bitdiff_doc   },
     {"bits2bytes", (PyCFunction) bits2bytes, METH_O,       bits2bytes_doc},
