@@ -36,40 +36,37 @@ from bitarray import (bitarray, frozenbitarray,
 tests = []
 
 
-def to_bytes(s):
-    if is_py3k:
-        return bytes(s.encode('latin1'))
-    else:
-        return bytes(s)  # which is str for Python 2
-
-def slicelen(s, length):
-    assert isinstance(s, slice)
-    start, stop, step = s.indices(length)
-    slicelength = (stop - start + (1 if step < 0 else -1)) // step + 1
-    if slicelength < 0:
-        slicelength = 0
-    return slicelength
-
-
 class Util(object):
 
-    def randombitarrays(self, start=1):
+    @staticmethod
+    def randombitarrays(start=1):
         for n in list(range(start, 25)) + [randint(1000, 2000)]:
             a = bitarray(endian=['little', 'big'][randint(0, 1)])
             a.frombytes(os.urandom(bits2bytes(n)))
             del a[n:]
             yield a
 
-    def randomlists(self):
+    @staticmethod
+    def randomlists():
         for n in list(range(25)) + [randint(1000, 2000)]:
             yield [bool(randint(0, 1)) for d in range(n)]
 
-    def rndsliceidx(self, length):
+    @staticmethod
+    def rndsliceidx(length):
         if randint(0, 1):
             return None
         else:
             return randint(-length-5, length+5)
 
+    @staticmethod
+    def slicelen(s, length):
+        assert isinstance(s, slice)
+        start, stop, step = s.indices(length)
+        slicelength = (stop - start + (1 if step < 0 else -1)) // step + 1
+        if slicelength < 0:
+            slicelength = 0
+        return slicelength
+
     def check_obj(self, a):
         self.assertEqual(repr(type(a)), "<class 'bitarray.bitarray'>")
         unused = 8 * a.buffer_info()[1] - len(a)
@@ -128,7 +125,7 @@ class TestsModuleFunctions(unittest.TestCase, Util):
         self.assertRaises(ValueError, bits2bytes, -924)
 
         self.assertEqual(bits2bytes(0), 0)
-        for n in range(1, 1000):
+        for n in range(1, 100):
             self.assertEqual(bits2bytes(n), (n - 1) // 8 + 1)
 
         for n, m in [(0, 0), (1, 1), (2, 1), (7, 1), (8, 1), (9, 2),
@@ -237,6 +234,13 @@ class CreateObjectTests(unittest.TestCase, Util):
             self.assertEqual(a, bitarray(lst))
             self.check_obj(a)
 
+    def test_iter3(self):
+        a = bitarray(itertools.repeat(False, 10))
+        self.assertEqual(a, bitarray(10 * '0'))
+        # Note that the through value of '0' is True: bool('0') -> True
+        a = bitarray(itertools.repeat('0', 10))
+        self.assertEqual(a, bitarray(10 * '1'))
+
     def test_01(self):
         a = bitarray('0010111')
         self.assertEqual(a.tolist(), [0, 0, 1, 0, 1, 1, 1])
@@ -252,14 +256,14 @@ class CreateObjectTests(unittest.TestCase, Util):
         self.assertRaises(ValueError, bitarray.__new__, bitarray, '01012100')
 
     def test_rawbytes(self): # this representation is used for pickling
-        for s, r in [('\x00', ''), ('\x07\xff', '1'), ('\x03\xff', '11111'),
-                     ('\x01\x87\xda', '10000111' '1101101')]:
-            self.assertEqual(bitarray(to_bytes(s), endian='big'),
+        for s, r in [(b'\x00', ''), (b'\x07\xff', '1'), (b'\x03\xff', '11111'),
+                     (b'\x01\x87\xda', '10000111' '1101101')]:
+            self.assertEqual(bitarray(s, endian='big'),
                              bitarray(r))
 
         for i in range(1, 8):
             self.assertRaises(ValueError, bitarray.__new__,
-                              bitarray, to_bytes(chr(i)))
+                              bitarray, bytes(bytearray([i])))
 
     def test_bitarray(self):
         for n in range(10):
@@ -302,19 +306,11 @@ tests.append(CreateObjectTests)
 
 class ToObjectsTests(unittest.TestCase, Util):
 
-    def test_int(self):
+    def test_numeric(self):
         a = bitarray()
-        self.assertRaises(TypeError, int, a)
-        if not is_py3k:
-            self.assertRaises(TypeError, long, a)
-
-    def test_float(self):
-        a = bitarray()
-        self.assertRaises(TypeError, float, a)
-
-    def test_complext(self):
-        a = bitarray()
-        self.assertRaises(TypeError, complex, a)
+        self.assertRaises(Exception, int, a)
+        self.assertRaises(Exception, float, a)
+        self.assertRaises(Exception, complex, a)
 
     def test_list(self):
         for a in self.randombitarrays():
@@ -374,7 +370,7 @@ class MetaDataTests(unittest.TestCase, Util):
 
 
     def test_length(self):
-        for n in range(1000):
+        for n in range(100):
             a = bitarray(n)
             self.assertEqual(len(a), n)
             self.assertEqual(a.length(), n)
@@ -483,7 +479,7 @@ class SliceTests(unittest.TestCase, Util):
             for dum in range(10):
                 step = self.rndsliceidx(la) or None
                 s = slice(self.rndsliceidx(la), self.rndsliceidx(la), step)
-                lb = randint(0, 10) if step is None else slicelen(s, la)
+                lb = randint(0, 10) if step is None else self.slicelen(s, la)
                 b = bitarray(lb)
                 c = bitarray(a)
                 c[s] = b
@@ -678,19 +674,19 @@ class MiscTests(unittest.TestCase, Util):
 
     def test_endianness1(self):
         a = bitarray(endian='little')
-        a.frombytes(to_bytes('\x01'))
+        a.frombytes(b'\x01')
         self.assertEqual(a.to01(), '10000000')
 
         b = bitarray(endian='little')
-        b.frombytes(to_bytes('\x80'))
+        b.frombytes(b'\x80')
         self.assertEqual(b.to01(), '00000001')
 
         c = bitarray(endian='big')
-        c.frombytes(to_bytes('\x80'))
+        c.frombytes(b'\x80')
         self.assertEqual(c.to01(), '10000000')
 
         d = bitarray(endian='big')
-        d.frombytes(to_bytes('\x01'))
+        d.frombytes(b'\x01')
         self.assertEqual(d.to01(), '00000001')
 
         self.assertEqual(a, c)
@@ -700,29 +696,29 @@ class MiscTests(unittest.TestCase, Util):
         a = bitarray(8, endian='little')
         a.setall(False)
         a[0] = True
-        self.assertEqual(a.tobytes(), to_bytes('\x01'))
+        self.assertEqual(a.tobytes(), b'\x01')
         a[1] = True
-        self.assertEqual(a.tobytes(), to_bytes('\x03'))
-        a.frombytes(to_bytes(' '))
-        self.assertEqual(a.tobytes(), to_bytes('\x03 '))
+        self.assertEqual(a.tobytes(), b'\x03')
+        a.frombytes(b' ')
+        self.assertEqual(a.tobytes(), b'\x03 ')
         self.assertEqual(a.to01(), '1100000000000100')
 
     def test_endianness3(self):
         a = bitarray(8, endian='big')
         a.setall(False)
         a[7] = True
-        self.assertEqual(a.tobytes(), to_bytes('\x01'))
+        self.assertEqual(a.tobytes(), b'\x01')
         a[6] = True
-        self.assertEqual(a.tobytes(), to_bytes('\x03'))
-        a.frombytes(to_bytes(' '))
-        self.assertEqual(a.tobytes(), to_bytes('\x03 '))
+        self.assertEqual(a.tobytes(), b'\x03')
+        a.frombytes(b' ')
+        self.assertEqual(a.tobytes(), b'\x03 ')
         self.assertEqual(a.to01(), '0000001100100000')
 
     def test_endianness4(self):
         a = bitarray('00100000', endian='big')
-        self.assertEqual(a.tobytes(), to_bytes(' '))
+        self.assertEqual(a.tobytes(), b' ')
         b = bitarray('00000100', endian='little')
-        self.assertEqual(b.tobytes(), to_bytes(' '))
+        self.assertEqual(b.tobytes(), b' ')
         self.assertNotEqual(a, b)
 
     def test_endianness5(self):
@@ -1004,12 +1000,19 @@ class BitwiseTests(unittest.TestCase, Util):
     def test_or(self):
         a = bitarray('11001')
         b = bitarray('10011')
+        aa = a.copy()
+        bb = b.copy()
         self.assertEQUAL(a | b, bitarray('11011'))
+        self.assertEQUAL(a, aa)
+        self.assertEQUAL(b, bb)
 
     def test_ior(self):
-        a =  bitarray('110010110')
-        a |= bitarray('100110011')
+        a = bitarray('110010110')
+        b = bitarray('100110011')
+        bb = b.copy()
+        a |= b
         self.assertEQUAL(a, bitarray('110110111'))
+        self.assertEQUAL(b, bb)
 
     def test_xor(self):
         a = bitarray('11001')
@@ -1413,7 +1416,7 @@ class MethodTests(unittest.TestCase, Util):
     def test_count2(self):
         for i in range(0, 256):
             a = bitarray()
-            a.frombytes(to_bytes(chr(i)))
+            a.frombytes(bytes(bytearray([i])))
             self.assertEqual(a.count(), a.to01().count('1'))
 
     def test_count3(self):
@@ -1709,7 +1712,7 @@ class MethodTests(unittest.TestCase, Util):
 
         for i in range(256):
             a = bitarray()
-            a.frombytes(to_bytes(chr(i)))
+            a.frombytes(bytes(bytearray([i])))
             aa = a.tolist()
             b = a
             b.bytereverse()
@@ -1726,23 +1729,22 @@ class StringTests(unittest.TestCase, Util):
 
     def randombytes(self):
         for n in range(1, 20):
-            yield to_bytes(''.join(chr(randint(0, 255))
-                                   for x in range(n)))
+            yield os.urandom(n)
 
     def test_frombytes(self):
         a = bitarray(endian='big')
-        a.frombytes(to_bytes('A'))
+        a.frombytes(b'A')
         self.assertEqual(a, bitarray('01000001'))
 
         b = a
-        b.frombytes(to_bytes('BC'))
+        b.frombytes(b'BC')
         self.assertEQUAL(b, bitarray('01000001' '01000010' '01000011',
                                      endian='big'))
         self.assertTrue(b is a)
 
         for b in self.randombitarrays():
             c = b.copy()
-            b.frombytes(to_bytes(''))
+            b.frombytes(b'')
             self.assertEQUAL(b, c)
 
         for b in self.randombitarrays():
@@ -1758,25 +1760,25 @@ class StringTests(unittest.TestCase, Util):
 
     def test_tobytes(self):
         a = bitarray()
-        self.assertEqual(a.tobytes(), to_bytes(''))
+        self.assertEqual(a.tobytes(), b'')
 
         for end in ('big', 'little'):
             a = bitarray(endian=end)
-            a.frombytes(to_bytes('foo'))
-            self.assertEqual(a.tobytes(), to_bytes('foo'))
+            a.frombytes(b'foo')
+            self.assertEqual(a.tobytes(), b'foo')
 
             for s in self.randombytes():
                 a = bitarray(endian=end)
                 a.frombytes(s)
                 self.assertEqual(a.tobytes(), s)
 
-        for n, s in [(1, '\x01'), (2, '\x03'), (3, '\x07'), (4, '\x0f'),
-                     (5, '\x1f'), (6, '\x3f'), (7, '\x7f'), (8, '\xff'),
-                     (12, '\xff\x0f'), (15, '\xff\x7f'), (16, '\xff\xff'),
-                     (17, '\xff\xff\x01'), (24, '\xff\xff\xff')]:
+        for n, s in [(1, b'\x01'), (2, b'\x03'), (3, b'\x07'), (4, b'\x0f'),
+                     (5, b'\x1f'), (6, b'\x3f'), (7, b'\x7f'), (8, b'\xff'),
+                     (12, b'\xff\x0f'), (15, b'\xff\x7f'), (16, b'\xff\xff'),
+                     (17, b'\xff\xff\x01'), (24, b'\xff\xff\xff')]:
             a = bitarray(n, endian='little')
             a.setall(1)
-            self.assertEqual(a.tobytes(), to_bytes(s))
+            self.assertEqual(a.tobytes(), s)
 
 
     def test_unpack(self):
@@ -1785,47 +1787,41 @@ class StringTests(unittest.TestCase, Util):
             self.assertIsInstance(a.unpack(), bytes)
         else:
             self.assertIsInstance(a.unpack(), str)
-        self.assertEqual(a.unpack(), to_bytes('\x00\xff'))
-        self.assertEqual(a.unpack(to_bytes('A')), to_bytes('A\xff'))
-        self.assertEqual(a.unpack(to_bytes('0'), to_bytes('1')),
-                         to_bytes('01'))
-        self.assertEqual(a.unpack(one=to_bytes('\x01')),
-                         to_bytes('\x00\x01'))
-        self.assertEqual(a.unpack(zero=to_bytes('A')),
-                         to_bytes('A\xff'))
-        self.assertEqual(a.unpack(one=to_bytes('t'), zero=to_bytes('f')),
-                         to_bytes('ft'))
+        self.assertEqual(a.unpack(), b'\x00\xff')
+        self.assertEqual(a.unpack(b'A'), b'A\xff')
+        self.assertEqual(a.unpack(b'0', b'1'), b'01')
+        self.assertEqual(a.unpack(one=b'\x01'), b'\x00\x01')
+        self.assertEqual(a.unpack(zero=b'A'), b'A\xff')
+        self.assertEqual(a.unpack(one=b't', zero=b'f'), b'ft')
 
-        self.assertRaises(TypeError, a.unpack,
-                          to_bytes('a'), zero=to_bytes('b'))
-        self.assertRaises(TypeError, a.unpack, foo=to_bytes('b'))
+        self.assertRaises(TypeError, a.unpack, b'a', zero=b'b')
+        self.assertRaises(TypeError, a.unpack, foo=b'b')
 
         for a in self.randombitarrays():
-            self.assertEqual(a.unpack(to_bytes('0'), to_bytes('1')),
-                             to_bytes(a.to01()))
+            self.assertEqual(a.unpack(b'0', b'1'), a.to01().encode())
 
             b = bitarray()
             b.pack(a.unpack())
             self.assertEqual(b, a)
 
             b = bitarray()
-            b.pack(a.unpack(to_bytes('\x01'), to_bytes('\x00')))
+            b.pack(a.unpack(b'\x01', b'\x00'))
             b.invert()
             self.assertEqual(b, a)
 
 
     def test_pack(self):
         a = bitarray()
-        a.pack(to_bytes('\x00'))
+        a.pack(b'\x00')
         self.assertEqual(a, bitarray('0'))
-        a.pack(to_bytes('\xff'))
+        a.pack(b'\xff')
         self.assertEqual(a, bitarray('01'))
-        a.pack(to_bytes('\x01\x00\x7a'))
+        a.pack(b'\x01\x00\x7a')
         self.assertEqual(a, bitarray('01101'))
 
         a = bitarray()
         for n in range(256):
-            a.pack(to_bytes(chr(n)))
+            a.pack(bytes(bytearray([n])))
         self.assertEqual(a, bitarray('0' + 255 * '1'))
 
         self.assertRaises(TypeError, a.pack, 0)
@@ -1852,9 +1848,8 @@ class FileTests(unittest.TestCase, Util):
     def test_pickle(self):
         for v in range(3):
             for a in self.randombitarrays():
-                fo = open(self.tmpfname, 'wb')
-                pickle.dump(a, fo, v)
-                fo.close()
+                with open(self.tmpfname, 'wb') as fo:
+                    pickle.dump(a, fo, v)
                 b = pickle.load(open(self.tmpfname, 'rb'))
                 self.assertFalse(b is a)
                 self.assertEQUAL(a, b)
@@ -1888,8 +1883,8 @@ class FileTests(unittest.TestCase, Util):
 
 
     def test_from_empty_file(self):
-        fo = open(self.tmpfname, 'wb')
-        fo.close()
+        with open(self.tmpfname, 'wb') as fo:
+            pass
 
         a = bitarray()
         a.fromfile(open(self.tmpfname, 'rb'))
@@ -1899,9 +1894,8 @@ class FileTests(unittest.TestCase, Util):
     def test_from_large_file(self):
         N = 100000
 
-        fo = open(self.tmpfname, 'wb')
-        fo.write(N * to_bytes('X'))
-        fo.close()
+        with open(self.tmpfname, 'wb') as fo:
+            fo.write(N * b'X')
 
         a = bitarray()
         a.fromfile(open(self.tmpfname, 'rb'))
@@ -1912,9 +1906,8 @@ class FileTests(unittest.TestCase, Util):
 
 
     def test_fromfile_Foo(self):
-        fo = open(self.tmpfname, 'wb')
-        fo.write(to_bytes('Foo\n'))
-        fo.close()
+        with open(self.tmpfname, 'wb') as fo:
+            fo.write(b'Foo\n')
 
         a = bitarray(endian='big')
         a.fromfile(open(self.tmpfname, 'rb'))
@@ -1940,62 +1933,54 @@ class FileTests(unittest.TestCase, Util):
     def test_fromfile_n(self):
         a = bitarray()
         a.fromstring('ABCDEFGHIJ')
-        fo = open(self.tmpfname, 'wb')
-        a.tofile(fo)
-        fo.close()
+        with open(self.tmpfname, 'wb') as fo:
+            a.tofile(fo)
 
         b = bitarray()
-        f = open(self.tmpfname, 'rb')
-        b.fromfile(f, 0);     self.assertEqual(b.tostring(), '')
-        b.fromfile(f, 1);     self.assertEqual(b.tostring(), 'A')
-        f.read(1)
-        b = bitarray()
-        b.fromfile(f, 2);     self.assertEqual(b.tostring(), 'CD')
-        b.fromfile(f, 1);     self.assertEqual(b.tostring(), 'CDE')
-        b.fromfile(f, 0);     self.assertEqual(b.tostring(), 'CDE')
-        b.fromfile(f);        self.assertEqual(b.tostring(), 'CDEFGHIJ')
-        b.fromfile(f);        self.assertEqual(b.tostring(), 'CDEFGHIJ')
-        f.close()
+        with open(self.tmpfname, 'rb') as f:
+            b.fromfile(f, 0);     self.assertEqual(b.tostring(), '')
+            b.fromfile(f, 1);     self.assertEqual(b.tostring(), 'A')
+            f.read(1)
+            b = bitarray()
+            b.fromfile(f, 2);     self.assertEqual(b.tostring(), 'CD')
+            b.fromfile(f, 1);     self.assertEqual(b.tostring(), 'CDE')
+            b.fromfile(f, 0);     self.assertEqual(b.tostring(), 'CDE')
+            b.fromfile(f);        self.assertEqual(b.tostring(), 'CDEFGHIJ')
+            b.fromfile(f);        self.assertEqual(b.tostring(), 'CDEFGHIJ')
 
         b = bitarray()
-        f = open(self.tmpfname, 'rb')
-        f.read(1);
-        self.assertRaises(EOFError, b.fromfile, f, 10)
-        f.close()
+        with open(self.tmpfname, 'rb') as f:
+            f.read(1);
+            self.assertRaises(EOFError, b.fromfile, f, 10)
+
         self.assertEqual(b.tostring(), 'BCDEFGHIJ')
 
         b = bitarray()
-        f = open(self.tmpfname, 'rb')
-        b.fromfile(f);
-        self.assertEqual(b.tostring(), 'ABCDEFGHIJ')
-        self.assertRaises(EOFError, b.fromfile, f, 1)
-        f.close()
+        with open(self.tmpfname, 'rb') as f:
+            b.fromfile(f);
+            self.assertEqual(b.tostring(), 'ABCDEFGHIJ')
+            self.assertRaises(EOFError, b.fromfile, f, 1)
 
 
     def test_tofile(self):
         a = bitarray()
-        f = open(self.tmpfname, 'wb')
-        a.tofile(f)
-        f.close()
+        with open(self.tmpfname, 'wb') as f:
+            a.tofile(f)
 
-        fi = open(self.tmpfname, 'rb')
-        self.assertEqual(fi.read(), to_bytes(''))
-        fi.close()
+        with open(self.tmpfname, 'rb') as fi:
+            self.assertEqual(fi.read(), b'')
 
         a = bitarray('01000110011011110110111100001010', endian='big')
-        f = open(self.tmpfname, 'wb')
-        a.tofile(f)
-        f.close()
+        with open(self.tmpfname, 'wb') as f:
+            a.tofile(f)
 
-        fi = open(self.tmpfname, 'rb')
-        self.assertEqual(fi.read(), to_bytes('Foo\n'))
-        fi.close()
+        with open(self.tmpfname, 'rb') as fi:
+            self.assertEqual(fi.read(), b'Foo\n')
 
         for a in self.randombitarrays():
             b = bitarray(a, endian='big')
-            fo = open(self.tmpfname, 'wb')
-            b.tofile(fo)
-            fo.close()
+            with open(self.tmpfname, 'wb') as fo:
+                b.tofile(fo)
 
             s = open(self.tmpfname, 'rb').read()
             self.assertEqual(len(s), a.buffer_info()[1])
@@ -2005,16 +1990,14 @@ class FileTests(unittest.TestCase, Util):
             self.assertRaises(TypeError, a.tofile)
             self.assertRaises(TypeError, a.tofile, StringIO())
 
-            f = open(self.tmpfname, 'wb')
-            a.tofile(f)
-            f.close()
+            with open(self.tmpfname, 'wb') as f:
+                a.tofile(f)
             self.assertRaises(TypeError, a.tofile, f)
 
         for n in range(20):
             a = n * bitarray('1', endian='little')
-            fo = open(self.tmpfname, 'wb')
-            a.tofile(fo)
-            fo.close()
+            with open(self.tmpfname, 'wb') as fo:
+                a.tofile(fo)
 
             s = open(self.tmpfname, 'rb').read()
             self.assertEqual(len(s), a.buffer_info()[1])
@@ -2282,29 +2265,30 @@ class BufferInterfaceTests(unittest.TestCase):
         v = memoryview(a)
         self.assertEqual(len(v), 3)
         self.assertEqual(v[0], 65 if is_py3k else 'A')
-        self.assertEqual(v[:].tobytes(), to_bytes('ABC'))
+        self.assertEqual(v.tobytes(), b'ABC')
         a[13] = 1
-        self.assertEqual(v[:].tobytes(), to_bytes('AFC'))
+        self.assertEqual(v.tobytes(), b'AFC')
 
     def test_read2(self):
-        a = bitarray([randint(0, 1) for d in range(8000)])
+        a = bitarray()
+        a.frombytes(os.urandom(100))
         v = memoryview(a)
-        self.assertEqual(len(v), 1000)
-        b = a[345 * 8 : 657 * 8]
-        self.assertEqual(v[345:657].tobytes(), b.tobytes())
-        self.assertEqual(v[:].tobytes(), a.tobytes())
+        self.assertEqual(len(v), 100)
+        b = a[34 * 8 : 67 * 8]
+        self.assertEqual(v[34:67].tobytes(), b.tobytes())
+        self.assertEqual(v.tobytes(), a.tobytes())
 
     def test_write(self):
-        a = bitarray(800000)
+        a = bitarray(8000)
         a.setall(0)
         v = memoryview(a)
         self.assertFalse(v.readonly)
-        v[50000] = 255 if is_py3k else '\xff'
-        self.assertEqual(a[399999:400009], bitarray('0111111110'))
-        a[400003] = 0
-        self.assertEqual(a[399999:400009], bitarray('0111011110'))
-        v[30001:30004] = to_bytes('ABC')
-        self.assertEqual(a[240000:240040].tobytes(), to_bytes('\x00ABC\x00'))
+        v[500] = 255 if is_py3k else '\xff'
+        self.assertEqual(a[3999:4009], bitarray('0111111110'))
+        a[4003] = 0
+        self.assertEqual(a[3999:4009], bitarray('0111011110'))
+        v[301:304] = b'ABC'
+        self.assertEqual(a[300 *8 : 305 * 8].tobytes(), b'\x00ABC\x00')
 
 if sys.version_info[:2] >= (2, 7):
     tests.append(BufferInterfaceTests)
@@ -2389,6 +2373,12 @@ tests.append(TestsFrozenbitarray)
 # ---------------------------------------------------------------------------
 
 def run(verbosity=1, repeat=1):
+    try:
+        import bitarray.test_util as btu
+        tests.extend(btu.tests)
+    except ImportError:
+        print("WARNING: could not import bitarray.test_util")
+
     print('bitarray is installed in: %s' % os.path.dirname(__file__))
     print('bitarray version: %s' % __version__)
     print('Python version: %s' % sys.version)
